
#include "Graph.hpp"
#include "gamex.hpp"

Graph::Graph(string _title, float _yaxisMin, float _yaxisMax, int _x1, int _y1, int _x2, int _y2)
{
	SetSize(_x1+10, _y1+10, _x2-10, _y2-10);
	numPoints = x2-x1;
	yaxisMin = _yaxisMin;
	yaxisMax = _yaxisMax;
	yaxisRange = yaxisMax - yaxisMin;

	minSeen = yaxisMax;
	maxSeen = yaxisMin;
	sprintf(txt_minmaxSeen, "min=%3.1f   max=%3.1f", minSeen, maxSeen);

	sprintf(txt_ymin, "%3.1f", yaxisMin);
	sprintf(txt_ymax, "%3.1f", yaxisMax);
	sprintf(txt_xmax, "%d", numPoints);
	sprintf(txt_title, "%s", _title.c_str());

	startIndex = 0;
	nextIndex = 0;
	showGraph = false;
	drawDeltaX = 1;
	timeout = false;
	ptBuffer = (float*) malloc(numPoints*sizeof(float)); 
	ptScreenBuffer = (int*) malloc(numPoints*sizeof(int)); 
}

Graph::~Graph()
{
	free(ptBuffer);
}

void Graph::SetSize(int _x1, int _y1, int _x2, int _y2)
{
	x1 = _x1;
	y1 = _y1;
	x2 = _x2;
	y2 = _y2;
	ymag = y2-y1;
	xmid = x1+(x2-x1)/2;
	ymid = y1+(y2-y1)/2;
};

void Graph::AddPoint(float pt)
{
	ptBuffer[nextIndex] = pt;

	if (!timeout)
	{
		if (nextIndex > 60)
			timeout = true;
	}

	int ypos = (int) (y1 + ((yaxisMax - pt) / yaxisRange) * ymag);
	if (ypos < y1)
		ptScreenBuffer[nextIndex] = y1;
	else if (ypos > y2)
		ptScreenBuffer[nextIndex] = y2-1;
	else
	{
		ptScreenBuffer[nextIndex] = ypos;
		if (pt > maxSeen && timeout)
		{
			maxSeen = pt;
			sprintf(txt_minmaxSeen, "min=%3.1f   max=%3.1f", minSeen, maxSeen);
		}
		if (pt < minSeen && timeout)
		{
			minSeen = pt;
			sprintf(txt_minmaxSeen, "min=%3.1f   max=%3.1f", minSeen, maxSeen);
		}
	}
	nextIndex = (nextIndex + 1) % numPoints;
	if (nextIndex == startIndex)
		startIndex = (startIndex + 1) % numPoints;
}

void Graph::ClearAll()
{
	startIndex = 0;
	nextIndex = 0;
}

void Graph::Draw()
{
	if (showGraph)
	{
		// Draw Points
		int prevX = x1;
		int prevY = ptScreenBuffer[startIndex];
		for (int i = (startIndex+1)%numPoints; i != nextIndex; i = (i+1)%numPoints, prevX++)
		{
			GameX.DrawLine(ColorX(0,255,0), prevX, prevY, prevX+1, ptScreenBuffer[i]);
			prevY =  ptScreenBuffer[i];
		}

		GameX.DrawText(xmid-40, y1+5, txt_minmaxSeen);
	}
	else
		GameX.DrawText(xmid-10, ymid, "DISABLED");

	// Draw Axis
	GameX.DrawLine(ColorX(0,0,255), x1, y2, x2, y2);		// x-axis
	GameX.DrawLine(ColorX(0,0,255), x1, y1, x1, y2);		// y-axis

	GameX.DrawText(x1-10, y1, txt_ymax);
	GameX.DrawText(x1-10, y2-12, txt_ymin);
	GameX.DrawText(x2-10, y2-10, txt_xmax);
	GameX.DrawText(xmid-10, y1-10, txt_title);
}
