#include "Point.hpp"

Point::Point() {
}

Point::Point(Vector3DF _pos, int _dist, Point *_parent, CameraX *_cam){

	// Create the image of a Dot:
	image.Create (128, 128, true); // 128x128 pixels, with transparency
	for (int px = 0 ; px < 128 ; px++)
		for (int py = 0 ; py < 128 ; py++)
			if (64.0f > sqrtf ((float) ((px-64)*(px-64)+(py-64)*(py-64))))
				image.SetPixel (px,py, px,py,px+py,192+py-px);

	// initialize position, velocity, acceleration vectors
	//pos.Set( 0.0f, 0.0f, 0.0f );
	pos = _pos;
	vel.Set( 0.0f, 0.0f, 0.0f );
	acc.Set( 0.0f, 0.0f, 0.0f );

	phi = 0;
	theta = 0;
	psi = 0;

	// initialize gravity vector to be initially up
	gravity.Set( ZERO_G, ZERO_G, ZERO_G );//G_OFFSET );

	prev_x = ZERO_G;

	if (_parent == NULL)
		parent = this;
	else
		parent = _parent;
	cam = _cam;
	dist = _dist;
}

void Point::Run( Vector3DF t, CameraX *c ){

	// grab cam object for projection view
	//cam = c[0];

	// update the acceleration vector based on input data
	//	and filter it
	acc = (t-gravity);

	// kill x-axis g-field acceleration
	//float gx = pow( pow(36,2)-pow(acc.y,2)-pow(acc.z,2), .5 );


	//float gx = 34 - pow( pow(acc.y,2)+pow(acc.z,2), .5 );
	//vel.x = gx;

	// compute angles and delta angles
	phi = atan2( acc.y, acc.z );		// phi = aten( y/z )
	theta = atan2( acc.z, acc.x );

	//vel.x = round( 33*cos(theta) );
	float gx = 33*cos(theta);
	vel.x = gx;
	acc.x = acc.x-gx;

	LPFilter( acc );

	psi += acc.x;

	// update position based upon angles
	pos.x = dist*cos( psi );
	pos.y = dist*cos( phi ) + dist*sin( psi );;
	pos.z = dist*sin( phi );

	Vector3DF d = pos - parent->pos;
	pos = parent->pos + d*(dist/d.Length());

	// move point if clicked on
	//ForceMove();

}

/* LPFs the acceleration vecotr */
void Point::LPFilter( Vector3DF &a ){

	if( abs(a.x) < 0 )
		a.x = 0;
	if( abs(a.y) < 2 )
		a.y = 0;
	if( abs(a.z) < 2 )
		a.z = 0;
}

float Point::round( float a ){

	// get decimal part
	//int b = a;
	//float decimal = a-(float)b;

	//if( abs(decimal) >= .5 )
	//	return ceil( a );
	//else
	//	return floor( a );

	if( a < 0 )
		return floor( a );
	else
		return ceil(a);

}

float Point::NormalizeAngle(float angle)
{
	float result = angle;

	if (angle > 2*PI)
		result = angle - 2*PI*((int)(0.5*angle/PI));
	else if (angle < 0)
		result = angle - 2*PI*((int)(0.5*angle/PI)-1);

	return result;
}

/* HPFs the velocity vecotr */
void Point::HPFilter( Vector3DF &v ){

	if( v.x > 20 )
		v.x = 20;
	else if( v.x < -20 )
		v.x = -20;

	if( v.y > 20 )
		v.y = 20;
	else if( v.y < -20 )
		v.y = -20;

	if( v.z > 20 )
		v.z = 20;
	else if( v.z < -20 )
		v.z = -20;
}

void Point::ForceMove(){
	
	// temporary variables
	bool moved_point = false;
	int mousex = GameX.GetMouseX();
	int mousey = GameX.GetMouseY();
	float mousedx = (float)GameX.GetMouseDX();
	float mousedy = (float)GameX.GetMouseDY();
	float x,y,z;

	// move dot if clicked on
	if( GameX.IsMouseDown(MOUSE_LEFT) ){

		// find p1 on screen space
		cam->Project( pos, x, y, z );

		// if within space and no previous point has been moved, move this point
		if( ( pow(x-mousex,2)+pow(y-mousey,2) ) <= 60 && !moved_point){	// p1 and mouse are close to each other

			moved_point = true;

			Vector3DF o = cam->GetDirVector();		// outward vector
			Vector3DF r = cam->GetSideVector();		// vector pointing to the right
			Vector3DF u = r; u.Cross(o);			// vector pointing up

			pos += ( r * Vector3DF(mousedx/2,mousedx/2,mousedx/2) );
			pos -= ( u * Vector3DF(mousedy/2,mousedy/2,mousedy/2) );
		}
	}
}

void Point::Draw(){

	// Draw the arm
	GameX.SetDrawScale (1.0f);
	GameX.DrawLine3D( ColorX(150,150,150), pos, parent->pos );

	// display the point
	GameX.SetDrawScale (5.0f);
	GameX.DrawImage3D ( &image, pos );

	// display the acceleration vector
	//GameX.DrawLine3D( ColorX(255,0,0), pos, acc+pos );

	// display the velocity vector
	//GameX.DrawLine3D( ColorX(0,255,0), pos, vel+pos );

}

void Point::Reset()
{
	pos.Set(0,0,0);
	vel.Set(0,0,0);
	acc.Set(0,0,0);
	phi = 0;
	theta = 0;
	psi = 0;
	prev_x = 0;
}