#include "SerialPort.hpp"

SerialPort::SerialPort()
{
}

SerialPort::~SerialPort()
{
	if(hSerialPort != INVALID_HANDLE_VALUE)
		CloseHandle(hSerialPort);
}

HRESULT SerialPort::Init(string portName)
{
	DWORD Success;
	DCB MyDCB = {0};
	LPCOMMTIMEOUTS MyCommTimeouts = (LPCOMMTIMEOUTS) malloc(sizeof(LPCOMMTIMEOUTS));
	HRESULT result = S_OK;

	try
	{
		hSerialPort = ::CreateFile(portName.c_str(),
			GENERIC_READ | GENERIC_WRITE, 0, 0, 
			OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);

		if ( hSerialPort == INVALID_HANDLE_VALUE )
			return E_FAIL;

		Success = GetCommState(hSerialPort, &MyDCB);
		if (!Success)
			return E_FAIL;

		MyDCB.DCBlength = sizeof(DCB);
		MyDCB.BaudRate = 9600;
		MyDCB.ByteSize = 8;
		MyDCB.Parity = NOPARITY;
		MyDCB.StopBits = ONESTOPBIT;
		MyDCB.fDsrSensitivity = 0;
		MyDCB.fDtrControl = DTR_CONTROL_ENABLE;
		MyDCB.fOutxDsrFlow = 0;
		MyDCB.ByteSize = 8;
		MyDCB.EofChar = 0;
		MyDCB.ErrorChar = 0;
		MyDCB.EvtChar = 0;
		MyDCB.fAbortOnError = 0;
		MyDCB.fBinary = 1;
		MyDCB.fDsrSensitivity = 0;
		MyDCB.fDummy2 = 0;
		MyDCB.fInX = 0;
		MyDCB.fNull = 0;
		MyDCB.fOutX = 0;
		MyDCB.fOutxCtsFlow = 1;
		MyDCB.fOutxDsrFlow = 0;
		MyDCB.wReserved = 0;
		MyDCB.wReserved1 = 0;
		MyDCB.XoffChar = 19;
		MyDCB.XoffLim = 512;
		MyDCB.XonChar = 17;
		MyDCB.XonLim = 2048;

		Success = SetCommState(hSerialPort, &MyDCB);
		if (!Success)
			return E_FAIL;

		Success = GetCommTimeouts(hSerialPort, MyCommTimeouts);
		if (!Success)
			return E_FAIL;

		MyCommTimeouts->ReadIntervalTimeout = MAXDWORD;
		MyCommTimeouts->ReadTotalTimeoutConstant = 1;
		MyCommTimeouts->ReadTotalTimeoutMultiplier = MAXDWORD;
		MyCommTimeouts->WriteTotalTimeoutConstant = 0;
		MyCommTimeouts->WriteTotalTimeoutMultiplier = 0;

		Success = SetCommTimeouts(hSerialPort, MyCommTimeouts);
		if (!Success)
			return E_FAIL;
	}
	catch(...)
	{
		result = E_FAIL;
	}

	return result;
}

HRESULT SerialPort::Read(BYTE* buffer, DWORD* BytesRead)
{
	DWORD Success;
	HRESULT result = S_OK;

	try
	{
		Success = ReadFile(hSerialPort, buffer, 255, BytesRead, NULL);
		if (!Success)
			result = E_FAIL;
	}
	catch(...)
	{
		result = E_FAIL;
	}
	return result;
}

HRESULT SerialPort::LastError(char* buffer)
{
	/*
	FormatMessage( 
		FORMAT_MESSAGE_ALLOCATE_BUFFER | 
		FORMAT_MESSAGE_FROM_SYSTEM | 
		FORMAT_MESSAGE_IGNORE_INSERTS,
		NULL,
		GetLastError(),
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		(LPTSTR) buffer,
		0,
		NULL 
	);
*/
	sprintf( buffer, "%d", GetLastError() );
	return S_OK;
}